<?php

class HomeUnit {
var $id;
var $name;

	public function __construct($id, $name = null) {
		$this->id = $id;
		$this->name = isset($name) ? $name : $id;
	}

	public function getRef() {
		return $this->name . " (id=" . $this->id . ")";
	}

}
			
class HomeDevice {


	public static function install() {
	}

	public static function listDevices($configData) {
		$output = array();
		foreach($configData as $configLine) {
			$cfg = explode(",", $configLine);
			if (count($cfg) > 3) {
				$id = $cfg[1] . $cfg[3];
				if (array_key_exists($id, $output)) {
					$output[$id] .= " / " . $cfg[0];
				} else {
					$output[$id] = $cfg[0];
				}
			}
		}
		//
		foreach($output as $line) {
			print $line . PHP_EOL;
		}
	}


	public function __construct() {
		$this->minbin = "/usr/local/minerva/bin";
	}

	public function processCommand($cmd, $unit, $param) {
		switch($cmd) {
			case 'init':
				return $this->init();

			case 'on':
				return $this->on($unit);

			case 'off':
				return $this->off($unit);

			case 'onifdark':
				return $this->onIfDark($unit);

			case 'onprofile':
				return $this->onProfile($unit, $param);

			case 'allon':
				return $this->allon($unit);

			case 'alloff':
				return $this->alloff($unit);

			case 'set':
				return $this->setBrightnessState($unit, $param);

			case 'color':
			case 'colour':
				$rgbColor = getColorRGB($param);
				return $this->setColorState($unit, $rgbColor);

			case 'get':
				return $this->showBrightnessState($unit);

			case 'ison':
				return $this->isOn($unit);

			case 'dim':
				return $this->dimBrightnessState($unit);

			case 'bright':
				return $this->raiseBrightnessState($unit);

			case 'status':
				return $this->status($unit);

		}
	}


	public function getIlluminationState($unit) {
		return false;
	}

	public function getBrightnessState($unit) {
		return 0;
	}

	public function init() {
	}


	public function status() {
	}


	public function on($unit) {
		exec($this->minbin . "/monexec x10control on '" . $unit->getRef() . "'");
		return $this->isOn($unit);
	}

	public function onIfDark($unit) {
		$sunlight = shell_exec($this->minbin . "/sunlight");
		if ($sunlight < 75) {
			$this->on($unit);
		}
	}

	public function onProfile($unit, $param) {
		if (!isset($param) || $param == "") {
			$param = "time";
		}
// TODO: profile includes 'energize' etc
		switch($param) {
			case "time":
				$value = date("H");
				// TODO: include minutes
				// TODO: have profile equations. e.g. 1800-2000:linear:10-50
				// TODO: All this needs to move
				// for the evening
				if ($value < 4) { 	// early morning. TODO. sunrise?
					$brightness = 20 + $value * 12;
				} else if ($value < 6) {
					$brightness = 20 + $value * 18;
				} else if ($value > 15) {
					$brightness = 100 - (24-$value) * 14;
				} else {
					$brightness = 100;
				}
				return $this->setBrightnessState($unit, $brightness);
//print $brightness . "  v = $value";
		}

	}

	public function off($unit) {
		exec($this->minbin . "/monexec x10control off '" . $unit->getRef() . "'");
		return $this->isOn($unit);
	}
	
	public function setColorState($rgbColor) {
	}

	public function allon($id) {
		exec($this->minbin . "/monexec x10control allon");
		print "All devices turned on" . PHP_EOL;
	}

	public function alloff() {
		exec($this->minbin . "/monexec x10control alloff");
		echo "All devices turned off";
	}

	public function showBrightnessState($unit) {
		$brightness = $this->getBrightnessState($unit);
		print "Level of " . $unit->getRef() . " is $brightness" . PHP_EOL;
		return ($brightness);
	}

	public function setBrightnessState($unit, $level) {
		return $this->showBrightnessState($unit);
	}

	public function dimBrightnessState($unit) {
		$current = $this->getBrightnessState($unit);
		$newBrightness = floor($current * 0.7);
		$this->setBrightnessState($unit, $newBrightness);
		return $this->showBrightnessState($unit);
	}

	public function raiseBrightnessState($unit) {
		$current = $this->getBrightnessState($unit);
		$newBrightness = max(10, floor($current * 1.5));
		$this->setBrightnessState($unit, $newBrightness);
		return $this->showBrightnessState($unit);
	}

	public function isOn($unit) {
		$name = $unit->getRef();

		if ($this->getIlluminationState($unit)) {
			print "$name is on" . PHP_EOL;
			return 1;
		} else {
			print "$name is off" . PHP_EOL;
			return 0;
		}
	}

}

?>
