<?php

class HomeDeviceHue extends HomeDevice {


	public function __construct($url, $key) {
		parent::__construct();
		$this->baseURL = "$url/api/";
		$this->url = "$url/api/$key/";
	}

	public function getIlluminationState($unit) {
		$s = ($this->hueGet($this->url . "lights/" . $unit->id));	
		$ison = $s->state->on;

		return $ison=="1" ? true : false;
	}

	public function getBrightnessState($unit) {
		$s = ($this->hueGet($this->url . "lights/" . $unit->id));	
		$bright = $s->state->bri;
		return floor(($bright * 100) / 255);
	}


	public function status() {
			$allLights = $this->hueGet($this->url . "/lights");
			print_r($allLights);
	}


	public function init() {
		$result = $this->huePost($this->baseURL, array('devicetype'=>'minerva'));

		if (isset($result[0]->success)) {
			$username = $result[0]->success->username;
			print "User key : $username" . PHP_EOL;
		} else {
			print "Error : " . $result[0]->error->description . PHP_EOL;
		}
	}

	public function refreshDevices() {
		$u = $this->url . "lights";
		if ($this->huePost($u)) {
		}
	}
	
	public function processCommand($cmd, $unit, $param) {
		switch($cmd) {
			case 'effect':
				return $this->effect($unit, 1);

			case 'none':
				return $this->effect($unit, 0);
		}

		return parent::processCommand($cmd, $unit, $param);
	}

	private function createCurl($url, $params, $header = "get") {
		$post_items = array();
		if ($params) {
			foreach ( $params as $key => $value) {
				$post_items[] = $key . '=' . $value;
			}
		}

		$post_string = implode ('&', $post_items);
		$tuCurl = curl_init(); 
		curl_setopt($tuCurl, CURLOPT_URL, $url);
		curl_setopt($tuCurl, CURLOPT_RETURNTRANSFER, 1);

		switch($header) {
			case 'put':
				curl_setopt($tuCurl, CURLOPT_CUSTOMREQUEST, "PUT");
			case 'post':
				curl_setopt($tuCurl, CURLOPT_POST, count($post_items));
				curl_setopt($tuCurl, CURLOPT_POSTFIELDS, json_encode($params));
				break;
		}
		curl_setopt($tuCurl, CURLOPT_HTTPHEADER, array("Content-Type: application/json"));
		$tuData = curl_exec($tuCurl); 
		curl_close($tuCurl); 
		
		return json_decode($tuData);
	}	

	public function hueGet($url, $params = "") {
		$curlResult = $this->createCurl($url, $params, "get");
		return $curlResult;
	}

	public function huePut($url, $params) {
		$curlResult = $this->createCurl($url, $params, "put");

		if (isset($curlResult[0]->success)) {
			return true;
		}
		$this->lastError = $curlResult[0];
		return false;
	}

	public function huePost($url, $params) {
		$curlResult = $this->createCurl($url, $params, "post");

		if (isset($curlResult[0]->success)) {
			return true;
		}
		return false;
	}

	public function effect($unit, $p) {

		$u = $this->url . "lights/" . $unit->id . "/state";
		$p = array("effect" => ($p ? "colorloop" : "none"));
		if ($this->huePut($u, $p)) {
			return parent::on($unit);
		}
		return -1;
	}

	public function on($unit) {

		$u = $this->url . "lights/" . $unit->id . "/state";
		$p = array("on" => true, "transitiontime" => 10);
		if ($this->huePut($u, $p)) {
			return parent::on($unit);
		}
		return -1;
	}

	public function off($unit) {
		$u = $this->url . "lights/" . $unit->id . "/state";
		$p = array("on" => false, "transitiontime" => 10);
		if ($this->huePut($u, $p)) {
			return parent::off($unit);
		}
		return -1;
	}

   public function setColorState($unit, $rgbColor) {
		$rgb = new colorRGBA($rgbColor);
		$hsb = $rgb->toHSB();

		$brightness = floor($hsb->b * 255);
		$saturation = floor($hsb->s * 255);
		$hue = floor($hsb->h * 65535);

		$u = $this->url . "lights/" . $unit->id . "/state";
		$p = array("bri" => $brightness, "sat" => $saturation, "hue" => $hue, "transitiontime" => 10);
		if ($this->huePut($u, $p)) {
			return parent::setColorState($unit, $rgbColor);
		}
		return -1;
   }

	public function allon($unit) {
#TODO
		$all = $this->hueGet($this->url . "lights/");
print_r($all);
		return parent::allon($unit);
	}

	public function alloff($unit) {
#TODO
		parent::alloff($unit);
	}

	public function setBrightnessState($unit, $level) {
		$huelevel = floor(($level * 255) / 100);
		$huelevel = min(max($huelevel, 0), 255);
		$this->on($unit);	// hue lights don't auto switch on 

		$u = $this->url . "lights/" . $unit->id . "/state";
		$p = array('bri'=>$huelevel);
		if ($this->huePut($u, $p)) {
			return parent::setBrightnessState($unit, $level);
		}
		return -1;
	}

# TODO: getcode getcodetype getcodelist getcodecount
 #  curl http://192.168.0.27/api/d7ae8b2151d50df1e61f380289f33bf/lights
}
?>
